package
{
	import org.flixel.*;

	public class Game
	{
		
		[Embed(source = "data/p1.png")] public static var ImgP1:Class;
		[Embed(source = "data/p2.png")] public static var ImgP2:Class;
		
		public static var levels:Array = [
			//level 0
				[	[1,1],
					[2,2]
				],
			//level1
				[	[2,2,2],
					[2,2,1]
				],								
			//level2
				[
					[2,3,3,3],
					[2,2,3,3]
				],
				//level3
				[
					[3,3,3,3,2],
					[3,3,3,4,3]
				],
				//level4
				[
					[3,3,3,3,2,4],
					[3,3,3,4,3,4]
				]
				
			];
		public static var width:int=4;
		public static var height:int=4;
		
		public static var cellsize:int=32;
		public static var margin:int=5;
		
		public var rects:Array;
		
		public var solutionh:Array;
		public var solutionv:Array;
		public var stateh:Array;
		public var statev:Array;
		
		private static function RectContains(rect:Array,x:int,y:int):Boolean
		{
			var l:int=rect[0];
			var t:int=rect[1];
			var r:int=l+rect[2]-1;
			var b:int=t+rect[3]-1;
			return x>=l && x<=r && y>=t && y<=b; 
		}
		
		public function RemoveRect(x:int,y:int):Game
		{
			var g:Game = Copy();
			for (var i:int=0;i<g.rects.length;i++)
			{
				var r:Array=g.rects[i];
				if (RectContains(r,x,y))
				{
					trace("removing rect");
					g.rects.splice(i,1);
					break;
				}
			}
			return g;
		}
		//[[1,2],[3,4]]
		private static function RectPoints(rect:Array):Array
		{
			var l:int=rect[0];
			var t:int=rect[1];
			var r:int=l+rect[2]-1;
			var b:int=t+rect[3]-1;
			var result:Array=[];
			for (var i:int=l;i<=r;i++)
			{
				for(var j:int=t;j<=b;j++)
				{
					result.push([i,j]);
				}
			}
			return result;
		}
		
		private static function RectsOverlap(rect1:Array,rect2:Array):Boolean
		{
			var points1:Array=RectPoints(rect1);
			for (var i:int=0;i<points1.length;i++)
			{
				var px:int=points1[i][0];
				var py:int=points1[i][1];
				if (RectContains(rect2,px,py))
					return true;
			}
			return false;
		}
		
		
		public function Occupied(x:int,y:int):Boolean
		{
			for(var i:int=0;i<rects.length;i++)
			{
				if (RectContains(rects[i],x,y))
					return true;
			}
			return false;
		}
		
		public function CanAddRect(rect:Array):Boolean
		{
			var points:Array = RectPoints(rect);
			for (var i:int=0;i<points.length;i++)
			{
				if (Occupied(points[i][0],points[i][1]))
					return false;
			}
			return true;
		}
		
		public function AddRect(rect:Array):Game
		{
			var g:Game = Copy();
			g.rects.push(rect.concat());
			return g;
		}
		
		public function Copy():Game
		{
			var g:Game = new Game();
			g.rects=[];
			for (var i:int=0;i<rects.length;i++)
			{
				g.rects.push(rects[i].concat());
			}
			return g;
		}
		
		public function Game()
		{
			rects=[];
			trace(FlxG);
			trace(FlxG.level);
			trace(Game.levels);
			trace(Game.levels[FlxG.level]);
			solutionh=Game.levels[FlxG.level][0];
			solutionv=Game.levels[FlxG.level][1];
			width=solutionh.length;
			height=solutionv.length;
			stateh=[];
			statev=[];
			var i:int;
			for (i=0;i<width;i++)
			{
				//solutionh.push(int(Math.random()*5));
				stateh.push(0);
			}
			for (i=0;i<height;i++)
			{
				//solutionv.push(int(Math.random()*5));
				statev.push(0);
			}
		}
		
		public var won:Boolean=false;
		
		private function RecalcStateLines():void
		{
			var i,j,k:int;
			//reset
			for (i=0;i<width;i++)
			{
				stateh[i]=0;
			}
			for (j=0;j<height;j++)
			{
				statev[j]=0;
			}
			
			//add to relevant rows for each rect
			for(i=0;i<rects.length;i++)
			{
				var r:Array=rects[i];
				//h
				for (j=r[0];j<r[0]+r[2];j++)
				{
					stateh[j]++;
				}
				//v
				for (j=r[1];j<r[1]+r[3];j++)
				{
					statev[j]++;
				}
			}
			
			won=true;
			for (i=0;i<width;i++)
			{
				if (stateh[i]!=solutionh[i])
				{
					won=false;
					break;
				}
			}
			for (j=0;j<height;j++)
			{
				if (statev[j]!=solutionv[j])
				{
					won=false;
					break;
				}
			}
		}
		
		public function Draw():FlxGroup
		{
			var result:FlxGroup = new FlxGroup();
			var i:int;
			var s:FlxSprite;
			var t:FlxText;
			
			//columns
			for (i=0;i<width+3;i++)
			{
				var top:int=margin+(i<2?2*cellsize:0);
				var bottom:int=margin+(height+2)*cellsize;
				var left:int=margin+i*cellsize;
				s = new FlxSprite(left,top);
				s.createGraphic(1,bottom-top,i<2?0xffaa4444:0xff000000);
				result.add(s);
			}
			
			//rows			
			for (i=0;i<height+3;i++)
			{
				left=margin+(i<2?2*cellsize:0);
				var right:int=margin+(width+2)*cellsize;
				top=margin+i*cellsize;
				
				s = new FlxSprite(left,top);
				s.createGraphic(right-left,1,i<2?0xffaa4444:0xff000000);
				result.add(s);
			}
			
			RecalcStateLines();
			
			//letters
			for (i=0;i<width;i++)
			{
				t = new FlxText(margin+(2+i)*cellsize+2,margin+2,cellsize-2,solutionh[i]);
				t.alignment="center";
				t.color=0x000000;
				result.add(t);
				
				t = new FlxText(margin+(2+i)*cellsize+2,margin+2+cellsize,cellsize-2,stateh[i]);
				t.alignment="center";
				t.color = stateh[i]==solutionh[i]?0x00ff00:0xff00ff;
				result.add(t);				
			}
			
			for (i=0;i<height;i++)
			{
				t = new FlxText(margin+2,margin+(2+i)*cellsize+2,cellsize-2,solutionv[i]);
				t.alignment="center";
				t.color=0x000000;
				result.add(t);
				
				t = new FlxText(margin+2+cellsize,margin+(2+i)*cellsize+2,cellsize-2,statev[i]);
				t.alignment="center";
				t.color = statev[i]==solutionv[i]?0x00ff00:0xff00ff;
				result.add(t);				
			}
			
			//rects.push([2,3,4,5]);
			//rects
			for(i=0;i<rects.length;i++)
			{
				var r:Array=rects[i];
				s = new FlxSprite(margin+(2+r[0])*cellsize+3,margin+(2+r[1])*cellsize+3);
				s.createGraphic(r[2]*cellsize-5,r[3]*cellsize-5,0xffffffff);
				s.color=0x7777ff;
				result.add(s);
			}
			
			if (won)
			{
				t= new FlxText(0-1,90,FlxG.width,"LEVEL COMPLETE\n\nclick to proceed");
				t.color=0xffffff;
				t.alignment="center";
				t.size=16;
				result.add(t);
				
				t= new FlxText(0+1,90,FlxG.width,"LEVEL COMPLETE\n\nclick to proceed");
				t.color=0xffffff;
				t.alignment="center";
				t.size=16;
				result.add(t);
				
				t= new FlxText(0,90-1,FlxG.width,"LEVEL COMPLETE\n\nclick to proceed");
				t.color=0xffffff;
				t.alignment="center";
				t.size=16;
				result.add(t);
				
				t= new FlxText(0,90+1,FlxG.width,"LEVEL COMPLETE\n\nclick to proceed");
				t.color=0xffffff;
				t.alignment="center";
				t.size=16;
				result.add(t);
				
				
				t= new FlxText(0,90,FlxG.width,"LEVEL COMPLETE\n\nclick to proceed");
				t.color=0x000000;
				t.alignment="center";
				t.size=16;
				result.add(t);
			}
			
			return result;
		}
	}
}